# TU TISS plugin for LDAP User Managment Tools

[ "$0" != "${BASH_SOURCE[0]}" ] || {
    echo "$0: this script must be sourced, not called" >&2
    exit 1
}

# plugin_tu_tiss_resolve <ret-aa-vn> <key> <key-type> <attribute> [..]
# resolves a person in TISS and returns a associative array with all
# requested attributes
# supported key types: oid mnr tiss_id tiss_url
# errors: 8, (key notfound (http 404),  91 (not all attributes found)
#         92 (resolver error)
function plugin_tu_tiss_resolve() {
    (( $# > 3 )) || bb_fatal "plugin_tu_tiss_resolve called with $#" \
	"instead of 4 or more args"

    local _ret_aa_vn="$1"
    local _key="$2"
    local _key_type="$3"
    shift 3

    local -n _ret_aa_ref="$_ret_aa_vn"

    local _jq=("jq" "--exit-status" "--raw-output")

    bb_vn_validate "$_ret_aa_vn" || \
	bb_fatal "plugin_tu_tiss_resolve called with illegal ret-aa-vn"

    # clear destination array
    _ret_aa_ref=()

    case "$_key_type" in
	mnr|oid) ;;
	tiss_id) _key_type="id";;
	tiss_url)
	    _key_type="id"
	    _key="${_key##*/}"
	    _key="${_key%%.*}"
	    ;;
	*)
	    bb_msg debug "plugin_tu_tiss_resolve called with unknown" \
		"key-type: $_key_type"
	    return 92
    esac

    local _retval
    local _answer
    local _api_url="https://tiss.tuwien.ac.at/api/person/v22/$_key_type/"
    _answer="$(curl --silent "$_api_url/$_key?intern=true")" || {
	bb_msg debug "plugin_tu_tiss_resolve failed to run:" \
	    "curl --silent \"$_api_url/$_key?intern=true\""
	return 92
    }

    if "${_jq[@]}" .error >/dev/null <<< "$_answer"
    then
        _retval="$?"
	(( _retval == 0 )) || {
	    bb_msg err "plugin_tu_tiss_resolve: jq failed with: $_retval"
	    return 92
	}

	local _http_status

	_http_status="$("${_jq[@]}" .error.httpStatus <<< "$_answer")"
	case "$_http_status" in
	    404) return 8;; # key not found
	    500)
		local _err_msg
		_err_msg="$("${_jq[@]}" .error.message <<< "$_answer"))" || \
		    _err_msg="UKNOWN"
		bb_msg err "plugin_tu_tiss_resolve: TISS REST API" \
		    "returned HTTP status 500 (Unerwarteter Serverfehler):" \
		    "$_err_msg"
		return  92
		;;
	    *)
		bb_msg alert "plugin_tu_tiss_resolve: TISS REST API" \
		    "returned HTTP status $_http_status, which is an" \
		    "undocumented API error"
		return 92
	esac
    else
	local _attr _val

	bb_msg debug "plugin_tu_tiss_resolve($_key_type=$_key):" \
	    "answer=$_answer"

	_retval=0
	for _attr in "$@"
	do
	    if _val="$("${_jq[@]}" ".$_attr" <<< "$_answer")"
	    then
		# shellcheck disable=SC1087 # FIXME
		_ret_aa_ref["$_attr"]="$_val"
	    else
		_retval=91
		bb_msg warning "plugin_tu_tiss_resolve: cound not find" \
		    "requested attribute: $_attr"
	    fi
	done

	return "$_retval"
    fi
}
